==> blobAndConquer-0.1-1.tar.gz: BlobAndConquer/src/explosions.cpp <==
/*
Copyright (C) 2005 Parallel Realities

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "explosions.h"

void explosionHarmUnit(Unit *unit, int radius, int distance, Vector position, Entity *owner)
{
	float damage = radius;
	
	if (distance > (damage / 10))
	{
		for (int i = radius ; i < distance ; i++)
		{
			damage *= 0.95;
		}
	}
	
	if (damage < 1)
	{
		return;
	}
	
	float power = (damage * 3) / 10;
	
	Math::limit(&power, 0.5, 3.0);
	
	bool wasAlive = (unit->health > 0);
	
	debug(("%s took %.2f damage from explosion\n", unit->getName(), damage));
	unit->applyDamage(damage);
	
	if (unit == player)
	{
		game->alterStat("TOTAL_DAMAGE_TAKEN", damage);
	}
	
	audio->playSound(SND_HIT, CH_ANY);
	
	if (unit->flags & EF_STATIC)
	{
		return;
	}
	
	if (owner != NULL)
	{
		if ((wasAlive) && (unit->health <= 0) && (unit->entityType == ET_BIOMECH))
		{
			switch (owner->entityType)
			{
				case ET_BLOB:
					if (owner == player)
					{
						game->alterStat("ENEMIES_KILLED", 1);
					}
					else
					{
						game->alterStat("ENEMIES_KILLED_TEEKA", 1);
					}
					break;
				
				case ET_BIOMECH:
					game->alterStat("ENEMIES_KILLED_FRIENDLY", 1);
					break;
			}
		}
	}
		
	Vector push = unit->position;
	push -= position;
	push.normalize();
	
	push.x *= (damage / 10);
	push.y *= (damage / 10);
	push.z *= (damage / 10);
	
	unit->velocity = push;
	
	unit->helpless = 100 + (power * 10);
	Math::addBit(&unit->flags, EF_BOUNCES);
	Math::limit(&unit->velocity.x, -2, 2);
	Math::limit(&unit->velocity.y, -2, 2);
	Math::limit(&unit->velocity.z, -2, 2);
}

void doExplosion(Vector position, int size, Entity *owner)
{
	audio->playSound(SND_GRENADE_EXPLODE, CH_EXPLODE);
	
	addExplosionParticles(size, position);
	
	float distance;
	
	for (Unit *unit = (Unit*)entityManager->enemyList.getFirstElement() ; unit != NULL ; unit = (Unit*)unit->next)
	{
		if (!unit->canBeDamaged())
		{
			continue;
		}
		
		distance = Math::getDistance(unit->position, position);
		
		if (distance > (size * 4))
		{
			continue;
		}
		
		explosionHarmUnit(unit, size, (int)distance, position, owner);
	}
	
	for (Unit *unit = (Unit*)entityManager->blobList.getFirstElement() ; unit != NULL ; unit = (Unit*)unit->next)
	{
		if (!unit->canBeDamaged())
		{
			continue;
		}
		
		distance = Math::getDistance(unit->position, position);
		
		if (distance > (size * 4))
		{
			continue;
		}
		
		explosionHarmUnit(unit, size, (int)distance, position, owner);
	}
	
	float damage;
	
	for (Entity *entity = (Entity*)entityManager->elementList.getFirstElement() ; entity != NULL ; entity = (Entity*)entity->next)
	{
		if (!(entity->flags & EF_IMMORTAL))
		{
			distance = Math::getDistance(entity->position, position);
			
			damage = (size * 4);
			
			if (distance > (size * 4))
			{
				continue;
			}
			
			if (distance > (damage / 10))
			{
				for (int i = (size * 4) ; i < distance ; i++)
				{
					damage *= 0.95;
				}
			}
			
			if (damage < 1)
			{
				return;
			}
			
			entity->health -= damage;
			
			debug(("Structure (%s) took %.2f damage from explosion\n", (entity->name != "" ? entity->getName() : "unnamed"), damage));
		}
	}
}

==> blobAndConquer-0.2-1.tar.gz: BlobAndConquer/src/explosions.cpp <==
/*
Copyright (C) 2005 Parallel Realities

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "explosions.h"

void explosionHarmUnit(Unit *unit, int radius, int distance, Vector position, Entity *owner)
{
	float damage = radius;
	
	if (distance > (damage / 10))
	{
		for (int i = radius ; i < distance ; i++)
		{
			damage *= 0.95;
		}
	}
	
	if (damage < 1)
	{
		return;
	}
	
	float power = (damage * 3) / 10;
	
	Math::limit(&power, 0.5, 3.0);
	
	bool wasAlive = (unit->health > 0);
	
	debug(("%s took %.2f damage from explosion\n", unit->getName(), damage));
	unit->applyDamage(damage, DAMAGE_EXPLOSION);
	
	if (unit == player)
	{
		game->alterStat("TOTAL_DAMAGE_TAKEN", damage);
	}
	
	audio->playSound(SND_HIT, CH_ANY);
	
	if (unit->flags & EF_STATIC)
	{
		return;
	}
	
	if ((unit == player) && (game->cheatStun))
	{
		return;
	}
	
	if (owner != NULL)
	{
		if ((wasAlive) && (unit->health <= 0) && (unit->entityType == ET_BIOMECH))
		{
			switch (owner->entityType)
			{
				case ET_BLOB:
					if (owner == player)
					{
						game->alterStat("ENEMIES_KILLED", 1);
					}
					else
					{
						game->alterStat("ENEMIES_KILLED_TEEKA", 1);
					}
					break;
				
				case ET_BIOMECH:
					game->alterStat("ENEMIES_KILLED_FRIENDLY", 1);
					break;
			}
		}
	}
		
	Vector push = unit->position;
	push -= position;
	push.normalize();
	
	push.x *= (damage / 10);
	push.y *= (damage / 10);
	push.z *= (damage / 10);
	
	unit->velocity = push;
	
	unit->helpless = 100 + (power * 10);
	Math::addBit(&unit->flags, EF_BOUNCES);
	Math::limit(&unit->velocity.x, -2, 2);
	Math::limit(&unit->velocity.y, -2, 2);
	Math::limit(&unit->velocity.z, -2, 2);
}

void doExplosion(Vector position, int size, Entity *owner)
{
	audio->playSound(SND_GRENADE_EXPLODE, CH_EXPLODE);
	
	addExplosionParticles(size, position);
	
	float distance;
	
	for (Unit *unit = (Unit*)entityManager->enemyList.getFirstElement() ; unit != NULL ; unit = (Unit*)unit->next)
	{
		if (!unit->canBeDamaged())
		{
			continue;
		}
		
		distance = Math::getDistance(unit->position, position);
		
		if (distance > (size * 4))
		{
			continue;
		}
		
		explosionHarmUnit(unit, size, (int)distance, position, owner);
	}
	
	for (Unit *unit = (Unit*)entityManager->blobList.getFirstElement() ; unit != NULL ; unit = (Unit*)unit->next)
	{
		if (!unit->canBeDamaged())
		{
			continue;
		}
		
		distance = Math::getDistance(unit->position, position);
		
		if (distance > (size * 4))
		{
			continue;
		}
		
		explosionHarmUnit(unit, size, (int)distance, position, owner);
	}
	
	float damage;
	
	for (Entity *entity = (Entity*)entityManager->structureList.getFirstElement() ; entity != NULL ; entity = (Entity*)entity->next)
	{
		if (!(entity->flags & EF_IMMORTAL))
		{
			distance = Math::getDistance(entity->position, position);
			
			damage = (size * 4);
			
			if (distance > (size * 4))
			{
				continue;
			}
			
			if (distance > (damage / 10))
			{
				for (int i = (size * 4) ; i < distance ; i++)
				{
					damage *= 0.95;
				}
			}
			
			if (damage < 1)
			{
				return;
			}
			
			entity->health -= damage;
			
			debug(("Structure (%s) took %.2f damage from explosion\n", (entity->name != "" ? entity->getName() : "unnamed"), damage));
		}
	}
}
